@@ -27,7 +27,7 @@
 The scale operation can also be done with subelements.
 
 The subelements operations only really work with polylines (Wires)
-because internally the functions `scaleVertex` and `scaleEdge`
+because internally the functions `scale_vertex` and `scale_edge`
 only work with polylines that have a `Points` property.
 """
 ## @package gui_scale
@@ -73,8 +73,7 @@ class Scale(gui_base_original.Modifier):
 
     def Activated(self):
         """Execute when the command is called."""
-        self.name = translate("draft", "Scale")
-        super(Scale, self).Activated(name=self.name)
+        super(Scale, self).Activated(name="Scale")
         if not self.ui:
             return
         self.ghosts = []
@@ -84,7 +83,7 @@ class Scale(gui_base_original.Modifier):
         """Get object selection and proceed if successful."""
         if Gui.Selection.getSelection():
             return self.proceed()
-        self.ui.selectUi()
+        self.ui.selectUi(on_close_call=self.finish)
         _msg(translate("draft", "Select an object to scale"))
         self.call = self.view.addEventCallback("SoEvent",
                                                gui_tool_utils.selectObject)
@@ -99,8 +98,8 @@ class Scale(gui_base_original.Modifier):
             groups.get_group_contents(self.selected_objects)
         self.selected_subelements = Gui.Selection.getSelectionEx()
         self.refs = []
-        self.ui.pointUi(self.name)
-        self.ui.modUi()
+        self.ui.pointUi(title=translate("draft",self.featureName), icon="Draft_Scale")
+        self.ui.isRelative.hide()
         self.ui.xValue.setFocus()
         self.ui.xValue.selectAll()
         self.pickmode = False
@@ -109,19 +108,25 @@ class Scale(gui_base_original.Modifier):
         _msg(translate("draft", "Pick base point"))
 
     def set_ghosts(self):
-        """Set the previews of the objects to scale."""
-        if self.ui.isSubelementMode.isChecked():
-            return self.set_subelement_ghosts()
-        self.ghosts = [trackers.ghostTracker(self.selected_objects)]
+        """Set the ghost to display."""
+        for ghost in self.ghosts:
+            ghost.remove()
+        if self.task and self.task.isSubelementMode.isChecked():
+            self.ghosts = self.get_subelement_ghosts()
+        else:
+            self.ghosts = [trackers.ghostTracker(self.selected_objects)]
 
-    def set_subelement_ghosts(self):
-        """Set the previews of the subelements from an object to scale."""
+    def get_subelement_ghosts(self):
+        """Get ghost for the subelements (vertices, edges)."""
         import Part
 
-        for object in self.selected_subelements:
-            for subelement in object.SubObjects:
-                if isinstance(subelement, (Part.Vertex, Part.Edge)):
-                    self.ghosts.append(trackers.ghostTracker(subelement))
+        ghosts = []
+        for sel in Gui.Selection.getSelectionEx("", 0):
+            for sub in sel.SubElementNames if sel.SubElementNames else [""]:
+                if "Vertex" in sub or "Edge" in sub:
+                    shape = Part.getShape(sel.Object, sub, needSubElement=True, retType=0)
+                    ghosts.append(trackers.ghostTracker(shape))
+        return ghosts
 
     def pickRef(self):
         """Pick a point of reference."""
@@ -186,12 +191,12 @@ class Scale(gui_base_original.Modifier):
         """Scale only the subelements if the appropriate option is set.
 
         The subelements operations only really work with polylines (Wires)
-        because internally the functions `scaleVertex` and `scaleEdge`
+        because internally the functions `scale_vertex` and `scale_edge`
         only work with polylines that have a `Points` property.
 
         BUG: the code should not cause an error. It should check that
         the selected object is not a rectangle or another object
-        that can't be used with `scaleVertex` and `scaleEdge`.
+        that can't be used with `scale_vertex` and `scale_edge`.
         """
         Gui.addModule("Draft")
         try:
@@ -233,7 +238,7 @@ class Scale(gui_base_original.Modifier):
         str_delta = DraftVecUtils.toString(self.delta)
         str_delta_corr = DraftVecUtils.toString(App.Vector(1,1,1) - self.delta)
 
-        _cmd = 'Draft.clone'
+        _cmd = 'Draft.make_clone'
         _cmd += '('
         _cmd += objects + ', '
         _cmd += 'forcedraft=True'
@@ -267,7 +272,7 @@ class Scale(gui_base_original.Modifier):
                 _cmd += ']'
                 arguments.append(_cmd)
         all_args = ', '.join(arguments)
-        command.append('Draft.copyScaledEdges([' + all_args + '])')
+        command.append('Draft.copy_scaled_edges([' + all_args + '])')
         command.append('FreeCAD.ActiveDocument.recompute()')
         return command
 
@@ -282,7 +287,7 @@ class Scale(gui_base_original.Modifier):
             for index, subelement in enumerate(obj.SubObjects):
                 if isinstance(subelement, Part.Vertex):
                     _vertex_index = int(obj.SubElementNames[index][V:]) - 1
-                    _cmd = 'Draft.scaleVertex'
+                    _cmd = 'Draft.scale_vertex'
                     _cmd += '('
                     _cmd += 'FreeCAD.ActiveDocument.'
                     _cmd += obj.ObjectName + ', '
@@ -293,7 +298,7 @@ class Scale(gui_base_original.Modifier):
                     command.append(_cmd)
                 elif isinstance(subelement, Part.Edge):
                     _edge_index = int(obj.SubElementNames[index][E:]) - 1
-                    _cmd = 'Draft.scaleEdge'
+                    _cmd = 'Draft.scale_edge'
                     _cmd += '('
                     _cmd += 'FreeCAD.ActiveDocument.'
                     _cmd += obj.ObjectName + ', '
@@ -331,10 +336,12 @@ class Scale(gui_base_original.Modifier):
                 bads.append(obj)
         if bads:
             if len(bads) == 1:
-                m = translate("draft", "Unable to scale object: ")
+                m = translate("draft", "Unable to scale object:")
+                m += " "
                 m += bads[0].Label
             else:
-                m = translate("draft", "Unable to scale objects: ")
+                m = translate("draft", "Unable to scale objects:")
+                m += " "
                 m += ", ".join([o.Label for o in bads])
             m += " - " + translate("draft","This object type cannot be scaled directly. Please use the clone method.")
             _err(m)
@@ -412,7 +419,7 @@ class Scale(gui_base_original.Modifier):
                     self.task.lock.setChecked(True)
                     self.task.setValue(d2/d1)
 
-    def finish(self, closed=False, cont=False):
+    def finish(self, cont=False):
         """Terminate the operation."""
         super(Scale, self).finish()
         for ghost in self.ghosts:
