@@ -21,7 +21,7 @@
 """Provides support for importing and exporting SVG files.
 
 It enables importing/exporting objects directly to/from the 3D document
-but doesn't handle the SVG output from the Drawing and TechDraw modules.
+but doesn't handle the SVG output from the TechDraw module.
 
 Currently it only reads the following entities:
 * paths, lines, circular arcs, rects, circles, ellipses, polygons, polylines.
@@ -38,7 +38,7 @@ Currently unsupported:
 
 __title__ = "FreeCAD Draft Workbench - SVG importer/exporter"
 __author__ = "Yorik van Havre, Sebastian Hoogen"
-__url__ = "https://www.freecadweb.org"
+__url__ = "https://www.freecad.org"
 
 # TODO:
 # ignoring CDATA
@@ -244,36 +244,46 @@ def getcolor(color):
     (r, g, b, a)
         RGBA float tuple, where each value is between 0.0 and 1.0.
     """
+    if color == "none":
+        FreeCAD.Console.PrintMessage("Color defined as 'none', defaulting to black\n")
+        return (0.0, 0.0, 0.0, 0.0)
     if color[0] == "#":
-        # Color string '#12ab9f'
-        if len(color) == 7:
+        if len(color) == 7 or len(color) == 9: # Color string '#RRGGBB' or '#RRGGBBAA'
             r = float(int(color[1:3], 16) / 255.0)
             g = float(int(color[3:5], 16) / 255.0)
-            b = float(int(color[5:], 16) / 255.0)
-        # Color string '#1af'
-        elif len(color) == 4:
+            b = float(int(color[5:7], 16) / 255.0)
+            a = 1.0
+            if len(color) == 9:
+                a = float(int(color[7:9], 16) / 255.0)
+                FreeCAD.Console.PrintMessage(f"Non standard color format #RRGGBBAA : {color}\n")
+            return (r, g, b, 1-a)
+        if len(color) == 4: # Color string '#RGB'
             # Expand the hex digits
             r = float(int(color[1], 16) * 17 / 255.0)
             g = float(int(color[2], 16) * 17 / 255.0)
             b = float(int(color[3], 16) * 17 / 255.0)
-        return (r, g, b, 0.0)
-    # Color string 'rgb(0.12,0.23,0.3,0.0)'
-    elif color.lower().startswith('rgb('):
-        cvalues = color[3:].lstrip('(').rstrip(')').replace('%', '').split(',')
-        if '%' in color:
-            r, g, b = [int(float(cv)) / 100.0 for cv in cvalues]
-        else:
-            r, g, b = [int(float(cv)) / 255.0 for cv in cvalues]
-        return (r, g, b, 0.0)
-    # Color string 'MediumAquamarine'
-    else:
-        v = svgcolorslower.get(color.lower())
-        if v:
-            r, g, b = [float(vf) / 255.0 for vf in v]
             return (r, g, b, 0.0)
-        # for k, v in svgcolors.items():
-        #    if k.lower() == color.lower():
-        #        pass
+    if color.lower().startswith('rgb(') or color.lower().startswith('rgba('): # Color string 'rgb[a](0.12,0.23,0.3,0.0)'
+        cvalues = color.lstrip('rgba(').rstrip(')').replace('%', '').split(',')
+        if len(cvalues) == 3:
+            a = 1.0
+            if '%' in color:
+                r, g, b = [int(float(cv)) / 100.0 for cv in cvalues]
+            else:
+                r, g, b = [int(float(cv)) / 255.0 for cv in cvalues]
+        if len(cvalues) == 4:
+            if '%' in color:
+                r, g, b, a = [int(float(cv)) / 100.0 for cv in cvalues]
+            else:
+                r, g, b, a = [int(float(cv)) / 255.0 for cv in cvalues]
+        return (r, g, b, 1-a)
+    # Trying named color like 'MediumAquamarine'
+    v = svgcolorslower.get(color.lower())
+    if v:
+        r, g, b = [float(vf) / 255.0 for vf in v]
+        return (r, g, b, 0.0)
+    FreeCAD.Console.PrintWarning(f"Unknown color format : {color} : defaulting to black\n")
+    return (0.0, 0.0, 0.0, 0.0)
 
 
 def transformCopyShape(shape, m):
@@ -643,6 +653,7 @@ class svgHandler(xml.sax.ContentHandler):
     """Parse SVG files and create FreeCAD objects."""
 
     def __init__(self):
+        super().__init__()
         """Retrieve Draft parameters and initialize."""
         _prefs = "User parameter:BaseApp/Preferences/Mod/Draft"
         params = FreeCAD.ParamGet(_prefs)
@@ -652,6 +663,7 @@ class svgHandler(xml.sax.ContentHandler):
         self.count = 0
         self.transform = None
         self.grouptransform = []
+        self.groupstyles = []
         self.lastdim = None
         self.viewbox = None
         self.symbols = {}
@@ -699,6 +711,7 @@ class svgHandler(xml.sax.ContentHandler):
         self.count += 1
         _msg('processing element {0}: {1}'.format(self.count, name))
         _msg('existing group transform: {}'.format(self.grouptransform))
+        _msg('existing group style: {}'.format(self.groupstyles))
 
         data = {}
         for (keyword, content) in list(attrs.items()):
@@ -717,7 +730,7 @@ class svgHandler(xml.sax.ContentHandler):
                 inks_doc_name = attrs.getValue('sodipodi:docname')
                 inks_full_ver = attrs.getValue('inkscape:version')
                 inks_ver_pars = re.search("\d+\.\d+", inks_full_ver)
-                if inks_ver_pars != None:
+                if inks_ver_pars is not None:
                     inks_ver_f = float(inks_ver_pars.group(0))
                 else:
                     inks_ver_f = 99.99
@@ -728,32 +741,41 @@ class svgHandler(xml.sax.ContentHandler):
                 else:
                     self.svgdpi = 96.0
             if 'inkscape:version' not in data:
-                _inf = ("This SVG file does not appear to have been produced "
-                        "by Inkscape. If it does not contain absolute units "
-                        "then a DPI setting will be used.")
-                _qst = ("Do you wish to use 96 dpi? Choosing 'No' "
-                        "will use the older standard 90 dpi.")
-                if FreeCAD.GuiUp:
-                    msgBox = QtGui.QMessageBox()
-                    msgBox.setText(translate("ImportSVG", _inf))
-                    msgBox.setInformativeText(translate("ImportSVG", _qst))
-                    msgBox.setStandardButtons(QtGui.QMessageBox.Yes
-                                              | QtGui.QMessageBox.No)
-                    msgBox.setDefaultButton(QtGui.QMessageBox.No)
-                    ret = msgBox.exec_()
-                    if ret == QtGui.QMessageBox.Yes:
-                        self.svgdpi = 96.0
-                    else:
-                        self.svgdpi = 90.0
-                    if ret:
-                        _msg(translate("ImportSVG", _inf))
-                        _msg(translate("ImportSVG", _qst))
-                        _msg("*** User specified {} "
-                             "dpi ***".format(self.svgdpi))
-                else:
+                # exact scaling is calculated later below. Here we just want
+                # to skip the DPI dialog if a unit is specified in the viewbox
+                if "width" in data and "mm" in attrs.getValue('width'):
                     self.svgdpi = 96.0
-                    _msg(_inf)
-                    _msg("*** Assuming {} dpi ***".format(self.svgdpi))
+                elif "width" in data and "in" in attrs.getValue('width'):
+                    self.svgdpi = 96.0
+                elif "width" in data and "cm" in attrs.getValue('width'):
+                    self.svgdpi = 96.0
+                else:
+                    _inf = ("This SVG file does not appear to have been produced "
+                            "by Inkscape. If it does not contain absolute units "
+                            "then a DPI setting will be used.")
+                    _qst = ("Do you wish to use 96 dpi? Choosing 'No' "
+                            "will use the older standard 90 dpi.")
+                    if FreeCAD.GuiUp:
+                        msgBox = QtGui.QMessageBox()
+                        msgBox.setText(translate("ImportSVG", _inf))
+                        msgBox.setInformativeText(translate("ImportSVG", _qst))
+                        msgBox.setStandardButtons(QtGui.QMessageBox.Yes
+                                                  | QtGui.QMessageBox.No)
+                        msgBox.setDefaultButton(QtGui.QMessageBox.No)
+                        ret = msgBox.exec_()
+                        if ret == QtGui.QMessageBox.Yes:
+                            self.svgdpi = 96.0
+                        else:
+                            self.svgdpi = 90.0
+                        if ret:
+                            _msg(translate("ImportSVG", _inf))
+                            _msg(translate("ImportSVG", _qst))
+                            _msg("*** User specified {} "
+                                 "dpi ***".format(self.svgdpi))
+                    else:
+                        self.svgdpi = 96.0
+                        _msg(_inf)
+                        _msg("*** Assuming {} dpi ***".format(self.svgdpi))
             if self.svgdpi == 1.0:
                 _wrn("This SVG file (" + inks_doc_name + ") "
                      "has an unrecognised format which means "
@@ -798,18 +820,16 @@ class svgHandler(xml.sax.ContentHandler):
                 if 'width' in data \
                         and 'height' in data \
                         and 'viewBox' in data:
-                    vbw = float(data['viewBox'][2])
-                    vbh = float(data['viewBox'][3])
-                    w = attrs.getValue('width')
-                    h = attrs.getValue('height')
-                    self.viewbox = (vbw, vbh)
                     if len(self.grouptransform) == 0:
                         unitmode = 'mm' + str(self.svgdpi)
                     else:
                         # nested svg element
                         unitmode = 'css' + str(self.svgdpi)
-                    abw = getsize(w, unitmode)
-                    abh = getsize(h, unitmode)
+                    vbw = getsize(data['viewBox'][2], 'discard')
+                    vbh = getsize(data['viewBox'][3], 'discard')
+                    abw = getsize(attrs.getValue('width'), unitmode)
+                    abh = getsize(attrs.getValue('height'), unitmode)
+                    self.viewbox = (vbw, vbh)
                     sx = abw / vbw
                     sy = abh / vbh
                     _data = data.get('preserveAspectRatio', [])
@@ -833,10 +853,10 @@ class svgHandler(xml.sax.ContentHandler):
                     m.scale(Vector(25.4/self.svgdpi, 25.4/self.svgdpi, 1))
             self.grouptransform.append(m)
         if 'fill' in data:
-            if data['fill'][0] != 'none':
+            if data['fill'] != 'none':
                 self.fill = getcolor(data['fill'])
         if 'stroke' in data:
-            if data['stroke'][0] != 'none':
+            if data['stroke'] != 'none':
                 self.color = getcolor(data['stroke'])
         if 'stroke-width' in data:
             if data['stroke-width'] != 'none':
@@ -856,6 +876,27 @@ class svgHandler(xml.sax.ContentHandler):
             self.color = self.col
             self.width = self.lw
 
+        # apply group styles
+        if name == "g":
+            self.groupstyles.append([self.fill, self.color, self.width])
+        if self.fill is None:
+            if "fill" not in data or data['fill'] != 'none':
+                # do not override fill if this item has specifically set a none fill
+                for groupstyle in reversed(self.groupstyles):
+                    if groupstyle[0] is not None:
+                        self.fill = groupstyle[0]
+                        break
+        if self.color is None:
+            for groupstyle in reversed(self.groupstyles):
+                if groupstyle[1] is not None:
+                    self.color = groupstyle[1]
+                    break
+        if self.width is None:
+            for groupstyle in reversed(self.groupstyles):
+                if groupstyle[2] is not None:
+                    self.width = groupstyle[2]
+                    break
+
         pathname = None
         if 'id' in data:
             pathname = data['id'][0]
@@ -883,7 +924,7 @@ class svgHandler(xml.sax.ContentHandler):
                 p2 = Vector(float(p2[0]), -float(p2[1]), 0)
                 p3 = data["freecad:dimpoint"]
                 p3 = Vector(float(p3[0]), -float(p3[1]), 0)
-                obj = Draft.makeDimension(p1, p2, p3)
+                obj = Draft.make_dimension(p1, p2, p3)
                 self.applyTrans(obj)
                 self.format(obj)
                 self.lastdim = obj
@@ -912,6 +953,8 @@ class svgHandler(xml.sax.ContentHandler):
                         sh = makewire(path)
                         if self.fill and sh.isClosed():
                             sh = Part.Face(sh)
+                            if sh.isValid() is False:
+                                sh.fix(1e-6, 0, 1)
                         sh = self.applyTrans(sh)
                         obj = self.doc.addObject("Part::Feature", pathname)
                         obj.Shape = sh
@@ -1184,6 +1227,8 @@ class svgHandler(xml.sax.ContentHandler):
                                 and len(sh.Wires) == 1 \
                                 and sh.Wires[0].isClosed():
                             sh = Part.Face(sh)
+                            if sh.isValid() is False:
+                                sh.fix(1e-6, 0, 1)
                         sh = self.applyTrans(sh)
                         obj = self.doc.addObject("Part::Feature", pathname)
                         obj.Shape = sh
@@ -1201,6 +1246,8 @@ class svgHandler(xml.sax.ContentHandler):
                 # sh = Part.Wire(path)
                 if self.fill and sh.isClosed():
                     sh = Part.Face(sh)
+                    if sh.isValid() is False:
+                        sh.fix(1e-6, 0, 1)
                 sh = self.applyTrans(sh)
                 obj = self.doc.addObject("Part::Feature", pathname)
                 obj.Shape = sh
@@ -1355,6 +1402,7 @@ class svgHandler(xml.sax.ContentHandler):
                     sh = self.applyTrans(sh)
                     obj = self.doc.addObject("Part::Feature", pathname)
                     obj.Shape = sh
+                    self.format(obj)
                     if self.currentsymbol:
                         self.symbols[self.currentsymbol].append(obj)
 
@@ -1492,6 +1540,8 @@ class svgHandler(xml.sax.ContentHandler):
         if name == "g" or name == "svg":
             _msg("closing group")
             self.grouptransform.pop()
+            if self.groupstyles:
+                self.groupstyles.pop()
         if name == "symbol":
             if self.doc.getObject("svgsymbols"):
                 group = self.doc.getObject("svgsymbols")
@@ -1590,14 +1640,15 @@ class svgHandler(xml.sax.ContentHandler):
                 cy = 0
                 angle = argsplit[0]
                 if len(argsplit) >= 3:
+                    # Rotate around a non-origin centerpoint (note: SVG y axis is opposite FreeCAD y axis)
                     cx = argsplit[1]
                     cy = argsplit[2]
-                    m.move(Vector(cx, -cy, 0))
+                    m.move(Vector(-cx, cy, 0)) # Reposition for rotation
                 # Mirroring one axis is equal to changing the direction
                 # of rotation
                 m.rotateZ(math.radians(-angle))
                 if len(argsplit) >= 3:
-                    m.move(Vector(-cx, cy, 0))
+                    m.move(Vector(cx, -cy, 0)) # Reverse repositioning
             elif transformation == 'skewX':
                 _m = FreeCAD.Matrix(1,
                                     -math.tan(math.radians(argsplit[0])))
@@ -1629,32 +1680,6 @@ class svgHandler(xml.sax.ContentHandler):
 # class svgHandler
 
 
-def decodeName(name):
-    """Decode encoded name.
-
-    Parameters
-    ----------
-    name : str
-        The string to decode.
-
-    Returns
-    -------
-    tuple
-    (string)
-        A tuple containing the decoded `name` in 'utf8', otherwise in 'latin1'.
-        If it fails it returns the original `name`.
-    """
-    try:
-        decodedName = (name.decode("utf8"))
-    except UnicodeDecodeError:
-        try:
-            decodedName = (name.decode("latin1"))
-        except UnicodeDecodeError:
-            _err("SVG error: couldn't determine character encoding")
-            decodedName = name
-    return decodedName
-
-
 def getContents(filename, tag, stringmode=False):
     """Get the contents of all occurrences of the given tag in the file.
 
