@@ -478,10 +478,10 @@ class ObjectDressup(object):
     def onDocumentRestored(self, obj):
         obj.setEditorMode("BoneBlacklist", 2)  # hide this one
 
-    def __getstate__(self):
+    def dumps(self):
         return None
 
-    def __setstate__(self, state):
+    def loads(self, state):
         return None
 
     def theOtherSideOf(self, side):
@@ -1320,10 +1320,10 @@ class ViewProviderDressup(object):
         panel.setupUi()
         return True
 
-    def __getstate__(self):
+    def dumps(self):
         return None
 
-    def __setstate__(self, state):
+    def loads(self, state):
         return None
 
     def onDelete(self, arg1=None, arg2=None):
